<?php
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Pragma: no-cache");
header('Content-Type: application/json; charset=utf-8');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204);
    header('Content-Length: 0');
    exit();
}

class RequestValidator {
    private static $requiredKeys = [
        'socks5_server_port', 'curl_connection_timeout',
        'curl_max_timeout', 'curl_http_method', 'curl_http_version',
        'curl_user_agent', 'link'
    ];

    public static function validateRequest($data) {
        foreach (self::$requiredKeys as $key) {
            if (!isset($data[$key]) || empty($data[$key])) {
                return self::errorResponse("Ошибка POST данные не корректны.");
            }
        }
        return true;
    }

    public static function validateSocksPort($value) {
        return self::validateNumeric($value, 1, 65535, "Ошибка значения socks5_server_port.");
    }

    public static function validateConnectionTimeout($value) {
        return self::validateNumeric($value, 1, 5, "Ошибка значения curl_connection_timeout.");
    }

    public static function validateMaxTimeout($value) {
        return self::validateNumeric($value, 1, 10, "Ошибка значения curl_max_timeout.");
    }

    public static function validateHttpMethod($value) {
        if (!in_array(strtolower($value), ['get', 'head'])) {
            return self::errorResponse("Ошибка значения curl_http_method.");
        }
        return true;
    }

    public static function validateHttpVersion($value) {
        if (!in_array($value, ['http1-0', 'http1-1', 'http2'])) {
            return self::errorResponse("Ошибка значения curl_http_version.");
        }
        return true;
    }

    public static function validateUserAgent($value) {
        return self::validateNumeric($value, 1, 10, "Ошибка значения curl_user_agent.");
    }

    public static function validateLink(&$value) {
        if (!preg_match('#^https?://#i', $value)) {
            $value = 'https://' . $value;
        }
        if (!filter_var($value, FILTER_VALIDATE_URL)) {
            return self::errorResponse("Ошибка значения link.");
        }
        return true;
    }

    private static function validateNumeric($value, $min, $max, $errorMessage) {
        if (!is_numeric($value) || $value < $min || $value > $max) {
            return self::errorResponse($errorMessage);
        }
        return true;
    }

    private static function errorResponse($message) {
        return [
            'result' => false,
            'message' => $message,
            'http_response_code' => '000'
        ];
    }
}

class CurlHandler {
    private static $userAgents = [
        1 => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/136.0.0.0 Safari/537.36 Edg/136.0.0.0',
        2 => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:138.0) Gecko/20100101 Firefox/138.0',
        3 => 'Mozilla/5.0 (Linux; Android 10; K) AppleWebKit/537.36 (KHTML, like Gecko) SamsungBrowser/28.0 Chrome/130.0.0.0 Mobile Safari/537.36'
    ];

    public static function executeRequest($data) {
        $ch = curl_init();
        
        curl_setopt_array($ch, [
            CURLOPT_URL => $data['link'],
            CURLOPT_PROXY => "127.0.0.1:".$data['socks5_server_port'],
            CURLOPT_PROXYTYPE => CURLPROXY_SOCKS5_HOSTNAME,
            CURLOPT_CONNECTTIMEOUT => $data['curl_connection_timeout'],
            CURLOPT_TIMEOUT => $data['curl_max_timeout'],
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HEADER => true,
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_SSL_VERIFYHOST => 2,
            CURLOPT_CAINFO => realpath(__DIR__ . '/curl_cert/cacert.pem')
        ]);

        if (strtolower($data['curl_http_method']) === 'head') {
            curl_setopt($ch, CURLOPT_NOBODY, true);
        }

        curl_setopt($ch, CURLOPT_HTTP_VERSION, self::getHttpVersion($data['curl_http_version']));

        curl_setopt($ch, CURLOPT_USERAGENT, self::$userAgents[$data['curl_user_agent']] ?? '');

        $response = curl_exec($ch);
        $error = curl_error($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if (!empty($error)) {
            return [
                'result' => false,
                'message' => "CURL ошибка: $error",
                'http_response_code' => '000'
            ];
        }

        return [
            'result' => true,
            'message' => 'OK',
            'http_response_code' => (string)$httpCode ?: '000'
        ];
    }

    private static function getHttpVersion($version) {
        switch ($version) {
            case 'http1-0': return CURL_HTTP_VERSION_1_0;
            case 'http1-1': return CURL_HTTP_VERSION_1_1;
            case 'http2':  return CURL_HTTP_VERSION_2;
            default:       return CURL_HTTP_VERSION_NONE;
        }
    }
}

try {
    $input = json_decode(file_get_contents('php://input'), true);
    
    $validationResult = RequestValidator::validateRequest($input);
    if ($validationResult !== true) {
        echo json_encode($validationResult, JSON_UNESCAPED_UNICODE);
        exit;
    }

    $validations = [
        'validateSocksPort' => $input['socks5_server_port'],
        'validateConnectionTimeout' => $input['curl_connection_timeout'],
        'validateMaxTimeout' => $input['curl_max_timeout'],
        'validateHttpMethod' => $input['curl_http_method'],
        'validateHttpVersion' => $input['curl_http_version'],
        'validateUserAgent' => $input['curl_user_agent'],
        'validateLink' => &$input['link']
    ];

    foreach ($validations as $method => $value) {
        $result = RequestValidator::$method($value);
        if ($result !== true) {
            echo json_encode($result, JSON_UNESCAPED_UNICODE);
            exit;
        }
    }

    $response = CurlHandler::executeRequest($input);
    echo json_encode($response, JSON_UNESCAPED_UNICODE);

} catch (Exception $e) {
    echo json_encode([
        'result' => false,
        'message' => 'Ошибка выполнения: ' . $e->getMessage(),
        'http_response_code' => '000'
    ], JSON_UNESCAPED_UNICODE);
}
?>