import { CONFIG, VALIDATION_RULES } from './prepare_config.js';

const MAIN_SERVERS_COUNT = 8;
const TEST_THREADS_COUNT = 40;
const REQUIRED_BLOCKS = ['block-3', 'block-4', 'block-5', 'block-6', 'block-7', 'block-8', 'block-9', 'block-10'];

const getDomElement = (id) => document.getElementById(id) || null;
const getValue = (obj, path) => path.split('.').reduce((acc, key) => acc?.[key], obj);

async function loadData() {
    try {
        logMessage('ИНФО', 'Начали загрузку данных...');
        const response = await fetch('./prepare.php?_=' + Date.now(), {
            headers: {
                'Accept-Charset': 'UTF-8',
                'Cache-Control': 'no-store, no-cache, must-revalidate',
                'Pragma': 'no-cache'
            }
        });
        
        if (!response.ok) {
            logMessage('ОШИБКА', `Ошибка загрузки prepare.php: ${response.result}`);
            return null;
        }
        
        const data = await response.json();
        
        if (!data || typeof data !== 'object') {
            logMessage('ОШИБКА', 'Некорректные данные от сервера.');
            return null;
        }
        
        logMessage('ИНФО', 'Завершено.');
        return data;
    } catch (error) {
        logMessage('ОШИБКА', `Ошибка загрузки данных: ${error.message}`);
        return null;
    }
}

function checkOS(data) {
    const os = data.os?.detected_os;
    
    if (!os) {
        logMessage('ОШИБКА', 'Не удалось определить систему.');
        return null;
    }

    const osElement = getDomElement('my-os');
    if (osElement) {
        osElement.textContent = `os: ${os}`;
    } else {
        logMessage('ОШИБКА', 'Элемент "my-os" не найден.');
    }
    
    if (os === 'unsupported') {
        logMessage('ОШИБКА', 'Неподдерживаемая система.');
        return null;
    }
    
    logMessage('ИНФО', `Определена система: ${os}`);
    return os;
}

function validateValue(key, value) {
    const rule = VALIDATION_RULES[key];
    return rule ? rule(value) : value != null && value !== '';
}

function validateConfig(data, os) {
    const config = CONFIG[os];
    
    if (!config) {
        logMessage('ОШИБКА', `Конфигурация для системы ${os} не найдена.`);
        return false;
    }

    const isValid = Object.entries(config).every(([key, description]) => {
        const value = getValue(data, key);
        const isValid = validateValue(key, value);
        const level = isValid ? 'ОТЛАДКА' : 'ОШИБКА';
        logMessage(level, `${description} -> ${isValid ? 'ОК' : 'ОШИБКА'}`);
        return isValid;
    });

    if (!isValid) logMessage('ОШИБКА', 'Обнаружены ошибки в конфигурации.');
    return isValid;
}

function updateUI_ADD_LINKS_TO_SELECT(data) {
    const selectLinks = document.getElementById('select-links');
    const linksConfig = data.config?.select_links || {};
    
    selectLinks.innerHTML = '';
    
    Object.keys(linksConfig).forEach(groupName => {
        const option = document.createElement('option');
        option.value = groupName;
        option.textContent = groupName;
        selectLinks.appendChild(option);
    });

    document.getElementById('add-links').addEventListener('click', () => {
        const selectedGroup = selectLinks.value;
        if (!linksConfig[selectedGroup]) {
            logMessage('ОШИБКА', `Группа "${selectedGroup}" не найдена в конфигурации.`);
            return;
        }

        const linksToAdd = linksConfig[selectedGroup];
        const linksTextarea = document.getElementById('links');
        const currentLinks = linksTextarea.value
            .split('\n')
            .filter(line => line.trim());
        
        linksToAdd.forEach(link => {
            currentLinks.push(link);
        });

        linksTextarea.value = currentLinks.join('\n');
        logMessage('ИНФО', `Добавлено ${linksToAdd.length} ссылок из группы "${selectedGroup}"`);
    });
}

function updateUI_MAKE_LINK_TO_PAC() {
    const pacInput = getDomElement('pac-link');
    if (!pacInput) {
        logMessage('ОШИБКА', 'Элемент "pac-link" не найден в DOM.');
        return;
    }
    
    if (!window.appData || !window.appData.config || !window.appData.config.local_ip) {
        logMessage('ОШИБКА', 'Данные для генерации PAC URL недоступны.');
        return;
    }

    const protocol = window.location.protocol;
    const localIP = window.appData.config.local_ip;
    const port = window.location.port || '80';
    const currentPath = window.location.pathname;
    const directoryPath = currentPath ? currentPath.substring(0, currentPath.lastIndexOf('/') + 1) : '/';
    
    const pacUrl = `${protocol}//${localIP}${port ? ':' + port : ''}${directoryPath}local.pac`;
    
    pacInput.value = pacUrl;
    logMessage('ОТЛАДКА', `Сгенерирован PAC URL: ${pacUrl}`);
}

document.addEventListener('DOMContentLoaded', () => {
    document.addEventListener('appDataReady', () => {
        updateUI_MAKE_LINK_TO_PAC();
    });
});

function updateUI_READ_DOMAINS(os, data) {
    const isWindows = os === 'windows';
    
    ['tfo', 'drop-sack', 'md5sig'].forEach(id => {
        const element = document.getElementById(id);
        if (element) element.hidden = isWindows;
    });

    for (let i = 1; i <= MAIN_SERVERS_COUNT; i++) {
        const serverKey = `main_server_${i}_hosts_file`;
        const serverData = data.files?.main_server_hosts?.[serverKey];
        const textarea = getDomElement(`my-server-${i}-links`);
        
        if (!textarea || !serverData) {
            logMessage('ОТЛАДКА', `Данные для (ByeDPI для использования ${i}) не найдены.`);
            continue;
        }

        const domains = (serverData.hosts || []).filter(d => d.trim());
        textarea.textContent = domains.join('\n');
        logMessage('ОТЛАДКА', `Домены для (ByeDPI для использования ${i}) получены: ${domains.length} записей.`);
    }
}

class ProcessCheckerBase {
    constructor(data, config) {
        this.data = data;
        this.config = config;
        const os = data.os?.detected_os || 'windows';
        this.processFile = os === 'windows' ? 'windows.php' : 'linux.php';
        this.ciadpiPath = data.files?.ciadpi_file?.filepath;
    }
    
    async checkAllServers() {
        if (!this.validatePrerequisites()) return false;

        const results = [];
        for (let i = 0; i < this.config.serversCount; i++) {
            const result = await this.checkServer(i + 1);
            results.push(result);
        }

        return results.every(result => result);
    }

    validatePrerequisites() {
        const errors = [];
        if (!this.processFile) errors.push('Файл проверки процессов не найден.');
        if (!this.ciadpiPath) errors.push('Путь к ciadpi не определен.');
        
        errors.forEach(error => logMessage('ОШИБКА', error));
        return errors.length === 0;
    }

    async checkServer(serverNumber) {
        try {
            const port = this.getPort(serverNumber);
            const response = await this.sendCheckRequest(port, this.config.action);
            if (!response.ok) {
                logMessage('ОШИБКА', `Ошибка сервера: ${response.result}`);
                return false;
            }
            const result = await response.json();
            
            return this.handleServerResponse(serverNumber, port, result);
        } catch (error) {
            logMessage('ОШИБКА', `Ошибка проверки сервера ${serverNumber}: ${error.message}`);
            return false;
        }
    }

    async sendCheckRequest(port, action) {
        return fetch(this.processFile + '?_=' + Date.now(), {
            method: 'POST',
            headers: { 
                'Content-Type': 'application/json; charset=UTF-8',
                'Cache-Control': 'no-cache',
                'Pragma': 'no-cache'
            },
            body: JSON.stringify({
                real_file_path: this.ciadpiPath,
                port: port,
                action: action
            })
        });
    }

    getPort(serverNumber) { throw new Error('Not implemented'); }
    handleServerResponse(serverNumber, port, result) { throw new Error('Not implemented'); }
}

class MainProcessChecker extends ProcessCheckerBase {
    constructor(data) {
        super(data, {
            serversCount: 8,
            portType: 'main',
            portPrefix: 'main',
            configPath: 'ciadpi_main_servers_tcp_ports',
            usageType: 'использования',
            action: 'check'
        });
        
        this.uiUpdater = new MainServerUIUpdater(data.config?.local_ip || '0.0.0.0');
    }

    getPort(serverNumber) {
        return this.data.config?.[this.config.configPath]?.[`${this.config.portPrefix}_${serverNumber}`];
    }

    handleServerResponse(serverNumber, port, result) {
        const logLevel = result.result ? 'ОТЛАДКА' : 'ОШИБКА';
        logMessage(logLevel, `(ByeDPI ${this.config.usageType} ${serverNumber}) -> TCP порт ${port}: ${result.message}`);

        if (result.result) {
            this.uiUpdater.updateSuccessfulServerUI(serverNumber, port, result);
        } else {
            this.uiUpdater.updateFailedServerUI(serverNumber);
        }

        return result.result;
    }
}

class TestProcessChecker extends ProcessCheckerBase {
    constructor(data) {
        super(data, {
            serversCount: 40,
            portType: 'test',
            portPrefix: 'thread',
            configPath: 'ciadpi_test_servers_tcp_ports',
            usageType: 'тестирования',
            action: 'check_and_kill'
        });
    }

    getPort(serverNumber) {
        return this.data.config?.[this.config.configPath]?.[`${this.config.portPrefix}_${serverNumber}`];
    }

    handleServerResponse(serverNumber, port, result) {
        const logLevel = result.result ? 'ОТЛАДКА' : 'ОШИБКА';
        logMessage(logLevel, `(ByeDPI ${this.config.usageType} ${serverNumber}) -> TCP порт ${port}: ${result.message}`);
        return result.result;
    }
}

class MainServerUIUpdater {
    constructor(localIP) {
        this.localIP = localIP;
    }

    updateSuccessfulServerUI(serverNumber, port, result) {
        this.updateServerInfo(serverNumber, port);
        
        const hasArguments = !!result.arguments;
        const hasHostsFile = !!result.hosts_file;
        const showStopButton = hasArguments || hasHostsFile;

        if (hasArguments) {
            this.updateCommandInput(serverNumber, result.arguments);
        }
        if (hasHostsFile) {
            this.updateHostsSelection(serverNumber, result.hosts_file);
        }

        this.toggleButtons(serverNumber, showStopButton);
    }

    updateFailedServerUI(serverNumber) {
        this.toggleButtons(serverNumber, false);
    }

    toggleButtons(serverNumber, showStopButton) {
        const startBtn = document.getElementById(`start-main-server-${serverNumber}`);
        const stopBtn = document.getElementById(`stop-main-server-${serverNumber}`);
        
        if (startBtn && stopBtn) {
            startBtn.hidden = showStopButton;
            stopBtn.hidden = !showStopButton;
        }
    }

    updateServerInfo(serverNumber, port) {
        const element = document.getElementById(`my-server-${serverNumber}-ip-and-port`);
        if (element) element.textContent = `${this.localIP}:${port}`;
    }

    updateCommandInput(serverNumber, commandArgs) {
        const input = document.getElementById(`my-server-${serverNumber}-command`);
        if (input && commandArgs) input.value = commandArgs;
    }

    updateHostsSelection(serverNumber, hostsFile) {
        const select = document.getElementById(`select-use-domains-list-or-not-${serverNumber}`);
        if (select) select.value = !!hostsFile ? 'true' : 'false';
    }
}

function toggleVisibilityBlocks(show) {
    REQUIRED_BLOCKS.forEach(blockId => {
        const block = getDomElement(blockId);
        if (block) block.hidden = !show;
    });
}

async function init() {
    try {
        const data = await loadData();
        if (!data) return;

        logMessage('ИНФО', 'Начали проверку конфигурации...');

        const os = checkOS(data);
        if (!os) {
            logMessage('ОШИБКА', 'Обнаружены ошибки.');
            return;
        }

        if (!validateConfig(data, os)) {
            logMessage('ОШИБКА', 'Обнаружены ошибки.');
            return;
        } else {
            logMessage('ИНФО', 'Завершено.');
        }

        logMessage('ИНФО', 'Начали проверку процессов и портов... (~1 минута)');

        const mainChecker = new MainProcessChecker(data);
        const testChecker = new TestProcessChecker(data);

        const mainCheckResult = await mainChecker.checkAllServers();
        const testCheckResult = await testChecker.checkAllServers();

        const allChecksSuccess = mainCheckResult && testCheckResult;

        if (allChecksSuccess) {
            window.appData = data;
            window.appDataReady = true;
            const event = new Event('appDataReady');
            document.dispatchEvent(event);
            
            toggleVisibilityBlocks(allChecksSuccess);
			
            updateUI_ADD_LINKS_TO_SELECT(data);       
            updateUI_READ_DOMAINS(os, data);
		    updateUI_MAKE_LINK_TO_PAC();			
			
            logMessage('ИНФО', 'Завершено.');
        } else {
            logMessage('ОШИБКА', 'Обнаружены ошибки.');
        }

    } catch (error) {
        logMessage('ОШИБКА', `Критическая ошибка инициализации: ${error.message}`);
        logMessage('ОШИБКА', 'Проверка прервана.');
    }
}

init();