class ServerStarter {
  constructor(serverNumber) {
    this.serverNumber = serverNumber;
    this.startButtonId = `start-main-server-${serverNumber}`;
    this.stopButtonId = `stop-main-server-${serverNumber}`;
    this.commandInputId = `my-server-${serverNumber}-command`;
    this.hostSelectId = `select-use-domains-list-or-not-${serverNumber}`;
  }

  init() {
    document.getElementById(this.startButtonId)?.addEventListener('click', () => this.handleStart());
  }

  async handleStart() {
    const startButton = document.getElementById(this.startButtonId);
    if (startButton) {
      startButton.disabled = true;
      startButton.classList.add('disabled');
    }

    try {
      const command = this.getSanitizedCommand();
      if (!this.validateCommand(command)) return;

      const { processFile, ciadpiPath, port } = this.getServerConfig();
      if (!this.validateConfig(processFile, ciadpiPath, port)) return;

      const postData = this.buildPostData(command, port, ciadpiPath);
      await this.sendStartRequest(postData, processFile);
    } finally {
      if (startButton) {
        startButton.disabled = false;
        startButton.classList.remove('disabled');
      }
    }
  }

  getProcessFile() {
    const osType = window.appData?.os?.detected_os;
    if (osType === "windows") {
      return window.appData?.files?.windows_file?.filename;
    } else if (osType === "linux") {
      return window.appData?.files?.linux_file?.filename;
    } else {
      return null;
    }
  }

  getServerConfig() {
    return {
      processFile: this.getProcessFile(),
      ciadpiPath: window.appData?.files?.ciadpi_file?.filepath,
      port: window.appData?.config?.ciadpi_main_servers_tcp_ports?.[`main_${this.serverNumber}`]
    };
  }

  validateConfig(processFile, ciadpiPath, port) {
    const errors = [];
    if (!processFile) errors.push('файл обработки');
    if (!ciadpiPath) errors.push('путь к ciadpi');
    if (!port) errors.push('порт');

    if (errors.length > 0) {
      logMessage('ОШИБКА', `(ByeDPI для использования ${this.serverNumber}) -> Отсутствует: ${errors.join(', ')}`);
      return false;
    }
    return true;
  }

  buildPostData(command, port, ciadpiPath) {
    const postData = {
      action: "check_and_start",
      real_file_path: ciadpiPath,
      port: port,
      arguments: command
    };

    const useHosts = document.getElementById(this.hostSelectId)?.value === 'true';
    if (useHosts) {
      const hostsFileKey = `main_server_${this.serverNumber}_hosts_file`;
      const hostsFilePath = window.appData?.files?.main_server_hosts?.[hostsFileKey]?.filepath;
      
      if (hostsFilePath) {
        postData.hosts_file_name = hostsFilePath;
      } else {
        logMessage('ОШИБКА', `Файл хостов для сервера ${this.serverNumber} не найден.`);
      }
    }

    return postData;
  }

  async sendStartRequest(postData, url) {
    logMessage('ИНФО', `(ByeDPI для использования ${this.serverNumber}) -> Запускаем...`);

    try {
      const controller = new AbortController();
      const timeoutId = setTimeout(() => controller.abort(), 10000);

      const response = await fetch(url, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'Accept-Charset': 'UTF-8',
          'Cache-Control': 'no-cache, no-store, must-revalidate',
          'Pragma': 'no-cache'
        },
        body: JSON.stringify(postData),
        signal: controller.signal
      });

      clearTimeout(timeoutId);
      await this.handleResponse(response);
    } catch (error) {
      this.handleRequestError(error);
    }
  }

  async handleResponse(response) {
    if (!response.ok) {
      logMessage('ОШИБКА', `HTTP ошибка! статус: ${response.result}`);
    }

    const data = await response.json();
    if (data?.result) {
      this.handleSuccess();
    } else {
      this.handleFailure(data?.message);
    }
  }

  handleSuccess() {
    logMessage('ИНФО', `(ByeDPI для использования ${this.serverNumber}) -> Запустилось.`);
    this.toggleButtons(true);
  }

  handleFailure(message) {
    const errorMessage = message || 'Неизвестная ошибка';
    logMessage('ОШИБКА', `(ByeDPI для использования ${this.serverNumber}) -> Запуск не удался: ${errorMessage}`);
    this.toggleButtons(false);
  }

  handleRequestError(error) {
    const errorMessage = error.name === 'AbortError' 
      ? 'Сработал таймаут' 
      : error.message;

    logMessage('ОШИБКА', `(ByeDPI для использования ${this.serverNumber}) -> Запуск не удался. ${errorMessage}`);
    this.toggleButtons(false);
  }

  toggleButtons(success) {
    const startButton = document.getElementById(this.startButtonId);
    const stopButton = document.getElementById(this.stopButtonId);
    
    if (startButton) {
      startButton.hidden = success;
      startButton.disabled = false;
      startButton.classList.remove('disabled');
    }
    if (stopButton) {
      stopButton.hidden = !success;
      stopButton.disabled = false;
      stopButton.classList.remove('disabled');
    }
  }

  getSanitizedCommand() {
    const commandInput = document.getElementById(this.commandInputId);
    return commandInput?.value?.trim() || '';
  }

  validateCommand(command) {
    if (!command) {
      logMessage('ОШИБКА', `(ByeDPI для использования ${this.serverNumber}) -> Команда не введена.`);
      return false;
    }
    return true;
  }
}

function initializeServerStarters() {
  for (let i = 1; i <= 8; i++) {
    new ServerStarter(i).init();
  }
}

document.addEventListener('DOMContentLoaded', () => {
  if (document.getElementById('start-main-server-1')) {
    initializeServerStarters();
  }
});