class ServerStopper {
  constructor(serverNumber) {
    this.serverNumber = serverNumber;
    this.startButtonId = `start-main-server-${serverNumber}`;
    this.stopButtonId = `stop-main-server-${serverNumber}`;
  }

  init() {
    document.getElementById(this.stopButtonId)?.addEventListener('click', () => this.handleStop());
  }

  async handleStop() {
    const stopButton = document.getElementById(this.stopButtonId);
    if (stopButton) {
      stopButton.disabled = true;
      stopButton.classList.add('disabled');
    }

    try {
      const { processFile, ciadpiPath, port } = this.getServerConfig();
      if (!this.validateConfig(processFile, ciadpiPath, port)) return;

      const postData = this.buildPostData(ciadpiPath, port);
      await this.sendStopRequest(postData, processFile);
    } finally {
      if (stopButton) {
        stopButton.disabled = false;
        stopButton.classList.remove('disabled');
      }
    }
  }

  getProcessFile() {
    const osType = window.appData?.os?.detected_os;
    if (osType === "windows") {
      return window.appData?.files?.windows_file?.filename;
    } else if (osType === "linux") {
      return window.appData?.files?.linux_file?.filename;
    } else {
      return null;
    }
  }

  getServerConfig() {
    return {
      processFile: this.getProcessFile(),
      ciadpiPath: window.appData?.files?.ciadpi_file?.filepath,
      port: window.appData?.config?.ciadpi_main_servers_tcp_ports?.[`main_${this.serverNumber}`]
    };
  }

  validateConfig(processFile, ciadpiPath, port) {
    const errors = [];
    if (!processFile) errors.push('файл обработки');
    if (!ciadpiPath) errors.push('путь к ciadpi');
    if (!port) errors.push('порт');

    if (errors.length > 0) {
      logMessage('ОШИБКА', `(ByeDPI для использования ${this.serverNumber}) -> Отсутствует: ${errors.join(', ')}`);
      return false;
    }
    return true;
  }

  buildPostData(ciadpiPath, port) {
    return {
      action: "check_and_kill",
      real_file_path: ciadpiPath,
      port: port
    };
  }

  async sendStopRequest(postData, url) {
    logMessage('ИНФО', `(ByeDPI для использования ${this.serverNumber}) -> Останавливаем...`);

    try {
      const controller = new AbortController();
      const timeoutId = setTimeout(() => controller.abort(), 10000);

      const response = await fetch(url, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'Accept-Charset': 'UTF-8',
          'Cache-Control': 'no-cache, no-store, must-revalidate',
          'Pragma': 'no-cache'
        },
        body: JSON.stringify(postData),
        signal: controller.signal
      });

      clearTimeout(timeoutId);
      await this.handleResponse(response);
    } catch (error) {
      this.handleRequestError(error);
    }
  }

  async handleResponse(response) {
    if (!response.ok) {
      logMessage('ОШИБКА', `HTTP ошибка! статус: ${response.result}`);
      return;
    }

    const data = await response.json();
    if (data?.result) {
      this.handleSuccess();
    } else {
      this.handleFailure(data?.message);
    }
  }

  handleSuccess() {
    logMessage('ИНФО', `(ByeDPI для использования ${this.serverNumber}) -> Остановилось.`);
    this.toggleButtons(true);
  }

  handleFailure(message) {
    const errorMessage = message || 'Неизвестная ошибка';
    logMessage('ОШИБКА', `(ByeDPI для использования ${this.serverNumber}) -> Остановка не удалась: ${errorMessage}`);
    this.toggleButtons(false);
  }

  handleRequestError(error) {
    const errorMessage = error.name === 'AbortError' 
      ? 'Сработал таймаут' 
      : error.message;

    logMessage('ОШИБКА', `(ByeDPI для использования ${this.serverNumber}) -> Остановка не удалась. ${errorMessage}`);
    this.toggleButtons(false);
  }

  toggleButtons(success) {
    const startButton = document.getElementById(this.startButtonId);
    const stopButton = document.getElementById(this.stopButtonId);
    
    if (startButton) {
      startButton.hidden = !success;
      startButton.disabled = false;
      startButton.classList.remove('disabled');
    }
    if (stopButton) {
      stopButton.hidden = success;
      stopButton.disabled = false;
      stopButton.classList.remove('disabled');
    }
  }
}

function initializeServerStoppers() {
  for (let i = 1; i <= 8; i++) {
    new ServerStopper(i).init();
  }
}

document.addEventListener('DOMContentLoaded', () => {
  if (document.getElementById('stop-main-server-1')) {
    initializeServerStoppers();
  }
});