<?php
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Pragma: no-cache");
header('Content-Type: application/json; charset=utf-8');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204);
    header('Content-Length: 0');
    exit();
}

class DomainValidator {
    public static function isValid(string $domain): bool {
        return (bool)preg_match('/^(?!-)(?:[A-Za-z0-9-]{1,63}\.){1,4}[A-Za-z]{2,6}(?<!-)$/', $domain);
    }

    public static function extractDomain(string $input): ?string {
        $input = trim($input);
        $parsed = @parse_url($input);
        if ($parsed && isset($parsed['host'])) {
            return $parsed['host'];
        }
        return $input ?: null;
    }
}

class FileHandler {
    public static function write(string $filePath, string $content): bool {
        return file_put_contents($filePath, $content) !== false;
    }

    public static function read(string $filePath): ?string {
        return file_exists($filePath) ? file_get_contents($filePath) : null;
    }

    public static function exists(string $filePath): bool {
        return file_exists($filePath);
    }
}

class Response {
    public static function error(string $message): array {
        return ['status' => false, 'message' => $message];
    }

    public static function success(string $message, array $data = []): array {
        return ['status' => true, 'message' => $message] + $data;
    }
}

class DomainProcessor {
    private int $server;
    private string $hostFile;
    private string $pacFile = 'local.pac';

    public function __construct(int $server) {
        $this->server = $server;
        $this->hostFile = "byedpi/main_server_{$server}_hosts.txt";
    }

    public function validateServer(): bool {
        return is_numeric($this->server) && $this->server >= 1 && $this->server <= 8;
    }

    public function processDomains(string $text): array {
        $text = trim($text);
        $lines = array_filter(array_map('trim', explode("\n", $text)));

        if (empty($lines)) {
            return $this->saveDomains([]);
        }

        $domains = array_map([DomainValidator::class, 'extractDomain'], $lines);
        $domains = array_filter($domains);
        $validDomains = array_filter($domains, [DomainValidator::class, 'isValid']);

        if (empty($validDomains)) {
            return Response::error("ОШИБКА: Все введённые строки не являются валидными доменами.");
        }

        return $this->saveDomains($validDomains);
    }

    private function saveDomains(array $validDomains): array {
        if (!FileHandler::exists($this->hostFile)) {
            return Response::error("ОШИБКА: Файл {$this->hostFile} не обнаружен.");
        }

        if (!FileHandler::write($this->hostFile, implode("\n", $validDomains))) {
            return Response::error("ОШИБКА: Не удалось записать в хост лист.");
        }

        return $this->updatePacFile($validDomains);
    }

    private function updatePacFile(array $validDomains): array {
        if (!FileHandler::exists($this->pacFile)) {
            return Response::error("ОШИБКА: Файл {$this->pacFile} не обнаружен.");
        }

        $pacContent = FileHandler::read($this->pacFile);
        if ($pacContent === null) {
            return Response::error("ОШИБКА: Не удалось прочитать PAC файл.");
        }

        $formattedDomains = array_map(function($domain) {
            return '            "' . $domain . '"';
        }, $validDomains);
        
        $newDomainsBlock = "[\n" . implode(",\n", $formattedDomains) . "\n        ]";

        $serverIndex = $this->server - 1;
        $pattern = '/"domains":\s*\[[^\]]*\]/';
        preg_match_all($pattern, $pacContent, $matches, PREG_OFFSET_CAPTURE);

        if (!isset($matches[0][$serverIndex])) {
            return Response::error("ОШИБКА: Сервер не найден в PAC файле.");
        }

        $newContent = substr_replace(
            $pacContent,
            '"domains": ' . $newDomainsBlock,
            $matches[0][$serverIndex][1],
            strlen($matches[0][$serverIndex][0])
        );

        if (!FileHandler::write($this->pacFile, $newContent)) {
            return Response::error("ОШИБКА: Не удалось записать в PAC файл.");
        }

        return Response::success(
            "ИНФО: Домены успешно записаны.", 
            ['cleaned' => $validDomains]
        );
    }
}

header('Content-Type: application/json; charset=UTF-8');
header('Cache-Control: no-cache, no-store, must-revalidate');
header('Pragma: no-cache');
header('Expires: 0');
$input = json_decode(file_get_contents('php://input'), true);

if ($input === null) {
    echo json_encode(Response::error('ОШИБКА: Не валидный JSON.'));
    exit;
}

$server = $input['server'] ?? null;
$text = $input['text'] ?? '';

$processor = new DomainProcessor($server);
if (!$processor->validateServer()) {
    echo json_encode(Response::error('ОШИБКА: Не валидный номер сервера.'));
    exit;
}

echo json_encode($processor->processDomains($text), JSON_UNESCAPED_UNICODE);